/*
    Aditi Raghavan
    Components needed: RFID RC522, Arduino Uno,
   Code for reading RFID's is heavily based on
   https://randomnerdtutorials.com/security-access-using-mfrc522-rfid-reader-with-arduino/
   Please note: There is interaction between the SPI library and SoftwareSerial library. You must use
   the sss (https://forum.arduino.cc/index.php?topic=251764) library for the Uno or altSoftwareSerial.


*/


#include <SPI.h>
#include <MFRC522.h>
#include <SoftwareSerial.h>
#include <Servo.h>
#define SS_PIN 10
#define RST_PIN 9

MFRC522 mfrc522(SS_PIN, RST_PIN);   // Create MFRC522 instance
Servo myDoor;

String tmpStr;
bool tmpbool = false; //If true, temporary pin will open door
bool isLocked = true;
const int servoPin = 6;
unsigned long curMillis = 0;
unsigned long prevSendMillis = 0;
unsigned long sendIntervalMillis = 4000;


void setup()
{
  sssBegin();
  Serial.begin(9600);   // Initiate a serial communication
  SPI.begin();      // Initiate  SPI bus
  mfrc522.PCD_Init();   // Initiate MFRC522
  myDoor.attach(servoPin);
  myDoor.write(90);
  Serial.print("Starting program..");
  Serial.println();

}
void loop()
{
  curMillis = millis();
  receiveMessage();
  if (mfrc522.PICC_IsNewCardPresent() && mfrc522.PICC_ReadCardSerial()) {
    //Show UID on serial monitor
    Serial.print("UID tag :");
    String content = "";
    byte letter;
    for (byte i = 0; i < mfrc522.uid.size; i++)
    {
      Serial.print(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " ");
      Serial.print(mfrc522.uid.uidByte[i], HEX);
      content.concat(String(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " "));
      content.concat(String(mfrc522.uid.uidByte[i], HEX));
    }
    Serial.print(".");
    // Serial.println();
    tmpStr.trim();
    String rfidUID = content.substring(1);
    rfidUID.toUpperCase();
    rfidUID.replace(" ", "");
    Serial.print("Message : ");
    content.toUpperCase();

    if (content.substring(1) == "7D C9 75 7E" ) //change here the UID of the card/cards that you want to give access
    {
      Serial.println("Authorized access");
      Serial.println();
      myDoor.write(10);
      isLocked = false;
    }
    else if (tmpStr == rfidUID && tmpbool)
    {
      Serial.println("Authorized temporary access");
      Serial.println();
      myDoor.write(10);
      isLocked = false;
    }

    else   {
      Serial.println(" Access denied");
      myDoor.write(90);
      delay(300);
      isLocked = true;
    }
  }
}

void receiveMessage() {
  if (sssAvailable() > 0) {
    byte bb = sssRead();
    //Serial.print((char) bb);
    if ( bb == '*') {
      tmpStr = "";
      return;
    }
    if ( bb == '?') {
      myDoor.write(90);
    }
    if ( bb == '+') {
      sendLocked(isLocked);
    }
    if (bb == '!') {
      tmpbool = !tmpbool;
      return;
    }
    if ( bb != '>') {
      Serial.print((char)bb);
      if (isAlphaNumeric((char)bb) || isWhitespace((char)bb)) {
        tmpStr += (char)bb;
      }
    }
    else if (bb == '>') {
      tmpbool = true;

    }
  }
}

void sendLocked(bool lock){
   if (curMillis - prevSendMillis >= sendIntervalMillis) {
      prevSendMillis += sendIntervalMillis;
      if(lock){
        sssWrite('y');
      }
      else { sssWrite('n');}
      return;
   }
}
